

BorderRing.DEFAULT_BACKGROUND_COLOR = NamedColor.getNamedColor("White", new Color("#ffffff"));


function BorderRing(widthIn, namedColorIn)
{
    var self = this;
    
    self.width = widthIn;
    self.namedColor = namedColorIn;

    
    
    /**
     * Instantiate BorderRing from XML document accessed by editor.
     * @param editor  XML document editor initialized with rosette XML doc
     */
    self.initializeFromXml = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor, extension)
    {
        
        var borderRingElement = editor.getElement(parent, "border_ring" + extension);
        
        self.width = editor.getDoubleElementValue(borderRingElement, "width", 0);
        
        // get ring color, with default of white if not specified in file
        var colorString = editor.getStringElementValue(borderRingElement, "ring_color", BorderRing.DEFAULT_BACKGROUND_COLOR.getARGBString());
        
        // convert from "aarrggbb" to "#rrggbb"
        colorString = NamedColor.argbToColorString(colorString);
        var ringColor = new Color(colorString);
        
        var ringColorName = editor.getAttribute(borderRingElement, "ring_color", "name", "");
        self.namedColor = NamedColor.getNamedColor(ringColorName, ringColor);
        
    }
    
    
    self.writeXML = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor, extension)
    {
        // create top-level ring node
        var borderRingElement = editor.appendElement(parent, "border_ring" + extension);
        
        // put in data
        editor.appendDoubleElement(borderRingElement, "width", self.width);
        
        var namedColorElement = editor.appendHexElement(borderRingElement, "ring_color", self.namedColor.getARGBString());
        namedColorElement.setAttribute("name", self.namedColor.getName());
        
    }
}
