

function DesignStateList(initialState, maxSizeIn = -1)
{
    
    var self = this;
    
    self.position = -1;
    self.designStates = new Array();
    
    // initialization at end... :-\
    //self.addDesignState(initialState);
    
    self.maxSize = maxSizeIn;
    
    

    /**
     * Return the element at previous position, and decrement position, but leave the element
     * on the list for possible "re-do" operations
     */
    self.getPreviousState = function()
    {
        if (self.position > 0)
        {
            self.position--;
            var state = self.designStates[self.position];
            
            //console.log("getPreviousState(): position = " + self.position + ", size = " + self.designStates.length + ", state = " + state.number);
            
            return state;
        }
        else
        {
            return null;
        }
        
    }
    
    
    /**
     * Support "re-do" operations by moving back up the list to the next previously "un-done" item
     * @return  Item previously "un-done"
     */
    self.getNextState = function()
    {
        if (self.position < self.designStates.length - 1)
        {
            self.position++;
            var state = self.designStates[self.position];
            
            //console.log("getNextState(): position = " + self.position + ", size = " + self.designStates.length + ", state = " + state.number);
            
            return state;
        }
        else
        {
            return null;
        }
    }
    
    
    self.addDesignState = function(state)
    {
        
        // clear any events above this in list - by adding a new event we
        // lose the ability to "re-do" further
        //self.designStates.splice(self.position + 1);
        self.designStates.length = self.position + 1;
        
        self.designStates.push(state);
        self.position++;
        
        // now see if we will exceed maximum size; if so, eliminate oldest state
        if ((self.maxSize > 0) && (self.position >= self.maxSize))
        {
            // remove head element
            self.designStates.shift();
            self.position--;
        }
        
        //console.log("addDesignState(): position = " + self.position + ", size = " + self.designStates.length + ", state = " + state.number);
    }
    
    
    self.clearStates = function()
    {
        self.designStates.length = 0;
        self.position = -1;
    }
    
    
    self.hasPreviousStates = function()
    {
        return (self.position > 0);
    }
    
    
    self.hasNextStates = function()
    {
        return (self.position < self.designStates.size() - 1);
    }
    
    
    self.setMaximumSize = function(maxSize)
    {
        self.maxSize = maxSize;
    }
    
    // initialization
    self.addDesignState(initialState);
    
    
}