


function PaletteManagerViewModel (viewId, colorChangeDispatcher)
{
    
    var self = this;
    
    self.viewID = viewId;
    self.view = document.getElementById(viewId);
    self.paletteTabList = getDescendantById(self.view, "PaletteTabs");
    self.paletteTabContentsDiv = getDescendantById(self.view, "PaletteTabContents");
    
    // passed to palettes for communicating color changes
    self.colorChangeDispatcher = colorChangeDispatcher;
    
    self.savePaletteButton = getDescendantById(self.view, "savePaletteButton");
    self.openPaletteInput = getDescendantById(self.view, "openPaletteInput");
    self.newPaletteButton = getDescendantById(self.view, "newPaletteButton");
    
    self.palettes = new Array();
    
    self.currentPalette = null;
    
    self.cumulativePaletteCount = 0;
    
    
    
    self.openPalette = function(event)
    {
        
        var file = event.target.files[0];
        var fileReader = new FileReader();
        fileReader.onload = function(event){self.createPaletteFromXml(file, event.target.result);};
        
        // set cursor to wait... doesn't work while Javascript runs...
        //document.body.style.cursor = "wait";
        
        fileReader.readAsText(file);
        
        // set cursor to default
        //document.body.style.cursor = "default";
    }
    
    self.openPaletteInput.addEventListener('change', self.openPalette);
    
    
    self.createPaletteFromXml = function(file, xmlString)
    {
        //console.log(xmlString);
        var fileName = file.name;
        
        // create and initialize a new palette with a unique name
        var paletteDivId = "Palette" + self.cumulativePaletteCount;
        self.cumulativePaletteCount++;
        
        var tabName = fileName;
        
        // strip off file type suffix, unless '.' is first character
        if (tabName.lastIndexOf('.') > 0)
            tabName = tabName.substring(0, tabName.lastIndexOf('.'));
        
        // create a tab and content div for the palette and add to document
        self.createTab(paletteDivId, tabName, false, true);      // not active, has close button
        
        
        try
        {
            // create PalletViewModel, which will add the palette contents; don't auto-add colors
            var palette = new PaletteViewModel(paletteDivId, colorChangeDispatcher, false);
            palette.id = paletteDivId;
            
            // get XML doc and editor for extracting palette
            var parser = new DOMParser();
            var xmlDoc = parser.parseFromString(xmlString,"text/xml");
            var editor = new XmlDocumentEditor(xmlDoc);
            var root = xmlDoc.documentElement;
            
            // initialize palette from XML doc
            palette.initializeFromXml(root, editor);
            
            // save the palette for when it's saved
            self.palettes.push(palette);
            
            // make active
            self.setActiveTab(palette.id);
            
        }
        catch (error)
        {
            window.alert("Problem reading palette file " + fileName + ": " + error.message);
            
            //clean up tab
            self.closeTab(paletteDivId);
        }
        
    }
    
    
    self.newPalette = function()
    {
        // create and initialize a new palette with a unique name
        var paletteDivId = "Palette" + self.cumulativePaletteCount;
        self.cumulativePaletteCount++;
        
        var tabName = paletteDivId;
        
        // create a tab and content div for the palette and add to document
        self.createTab(paletteDivId, tabName, false, true);      // not active, has close button
        
        // create PalletViewModel, which will add the palette contents; don't auto-add colors
        var palette = new PaletteViewModel(paletteDivId, colorChangeDispatcher, false);
        palette.id = paletteDivId;
        
        // save the palette for when it's saved
        self.palettes.push(palette);
        
        // make active
        self.setActiveTab(palette.id);
    }
    
    self.newPaletteButton.addEventListener('click', self.newPalette);
    
    
    
    self.findPaletteById = function(paletteId)
    {
        for (var i = 0; i < self.palettes.length; i++)
        {
            if (self.palettes[i].id == paletteId)
                return self.palettes[i];
        }
        
        return null;
    }
    
    
    self.setCurrentPalette = function(paletteId)
    {
        var currentPaletteId = paletteId;
        self.currentPalette = self.findPaletteById(currentPaletteId);
    }
    
    
    
    self.closeTab = function(paletteId)
    {
        // remove tab and palette from HTML doc
        var tab = getChildById(self.paletteTabList, paletteId + "Tab");
        self.paletteTabList.removeChild(tab);
        
        var paletteDiv = getChildById(self.paletteTabContentsDiv, paletteId);
        self.paletteTabContentsDiv.removeChild(paletteDiv);
        
        // remove palette from palette list
        var palette = self.findPaletteById(paletteId);
        var paletteIndex = self.palettes.indexOf(palette);
        if (paletteIndex >= 0)
            self.palettes.splice(paletteIndex, 1);
        
        // if this was active, set active palette to one before it in palette list
        if (palette == self.currentPalette)
        {
            self.setActiveTab(self.palettes[paletteIndex - 1].id);
        }
        
    }
    
    
    self.setActiveTab = function(paletteId)
    {
        self.currentPalette = self.findPaletteById(paletteId);
        
        // just click the link associated with the tab - activation and whatnot
        // handled automatically
        var tab = getChildById(self.paletteTabList, paletteId + "Tab");
        var link = getChildById(tab, paletteId + "TabLink");
        link.click();
    }
    
    
    
    self.createTab = function(paletteDivId, paletteTitle, setActive, hasCloseButton)
    {
        // create list item for tab
        var tab = document.createElement("li");
        tab.setAttribute("class", "nav-item");
        tab.setAttribute("id", paletteDivId + "Tab");
        
        // create link to palette panel
        var link = document.createElement("a");
        link.setAttribute("id", paletteDivId + "TabLink");
        link.setAttribute("data-toggle", "tab");
        link.setAttribute("role", "tab");
        link.setAttribute("paletteId", paletteDivId);
        link.setAttribute("href", "#" + paletteDivId);
        if (setActive)
            link.setAttribute("class", "nav-link flex-sm-fill text-sm-center active");
        else
            link.setAttribute("class", "nav-link flex-sm-fill text-sm-center");
        link.addEventListener("click", function() {self.setCurrentPalette(paletteDivId)});
        
        // create tab title as text node, and append to link
        var tabTitle = document.createTextNode(paletteTitle);
        
        
        
        // attach to document
        self.paletteTabList.appendChild(tab);
        tab.appendChild(link);
        link.appendChild(tabTitle);
        
        if (hasCloseButton)
        {
            var closeButton = document.createElement("button");
            closeButton.setAttribute("id", paletteDivId + "CloseButton");
            closeButton.setAttribute("class", "btn close close-button");
            closeButton.setAttribute("paletteId", paletteDivId);
            
            // add stopPropagation so clicking close box doesn't also result in (removed) tab being activated through propagated (bubbled?) link click
            closeButton.addEventListener("click", function(event) {self.closeTab(paletteDivId); event.stopPropagation();});
            
            closeButton.appendChild(document.createTextNode("X"));
            link.appendChild(closeButton);
        }
        
        
        // now create tab contents div for the palette to populate
        var paletteDiv = document.createElement("div");
        paletteDiv.setAttribute("id", paletteDivId);
        paletteDiv.setAttribute("role", "tabpanel");
        if (setActive)
            paletteDiv.setAttribute("class", "tab-pane fade in show active");
        else
            paletteDiv.setAttribute("class", "tab-pane fade in");
        
        self.paletteTabContentsDiv.appendChild(paletteDiv);
        
    }
    
    
    self.saveCurrentPalette = function(event)
    {
        var xmlString = vkbeautify.xml(self.currentPalette.generateXmlString());
        var blob = new Blob([xmlString], {type: "text/plain;charset=utf-8"});
        saveAs(blob, "palette.xml");
    }
    
    
    self.savePaletteButton.addEventListener('click', self.saveCurrentPalette);
    
    
    
    
    // initialization
    {
        // add the CurrentColors tab as first
        
        // create and initialize a new palette with a unique name      
        var paletteDivId = "Palette" + self.cumulativePaletteCount;
        self.cumulativePaletteCount++;
        
        // create a tab and content div for the palette and add to document
        self.createTab(paletteDivId, "Current Colors", true, false);    // active, no close button
        
        // create PalletViewModel, which will add the palette contents
        var currentColorsPalette = new PaletteViewModel(paletteDivId, colorChangeDispatcher, true);
        currentColorsPalette.id = paletteDivId;
        
        // save the palette
        self.palettes.push(currentColorsPalette);
        self.currentPalette = currentColorsPalette;
    }
    
    
    
    /*
    $('a[data-toggle="tab"]').on('show.bs.tab', function (e) {
        self.currentTab = e.target; // newly activated tab
        // e.relatedTarget // previous active tab
      });
      
      
        // Add a Tab
        $('#btnAddPage').click(function() 
        {
            pageNum++;
            $('#pageTab').append(
                $('<li><a href="#page' + pageNum + '">' + 'Page ' + pageNum + '<button class="close" type="button" ' + 'title="Remove this page">×</button>' + '</a></li>'));
    
            $('#pageTabContent').append(
                $('<div class="tab-pane" id="page' + pageNum +
                '">Content page' + pageNum + '</div>'));
    
            $('#page' + pageNum).tab('show');
        }
        

    // Remove a Tab

    $('#pageTab').on('click', ' li a .close', function() {
    var tabId = $(this).parents('li').children('a').attr('href');
    $(this).parents('li').remove('li');
    $(tabId).remove();
    reNumberPages();
    $('#pageTab a:first').tab('show');
    });

    */
    

}