

function PaletteViewModel (viewId, colorChangeDispatcher, autoAddColors)
{
	var self = this;
	
	// create divs for color and control buttons
	self.viewId = viewId;
	self.view = document.getElementById(viewId);
	
	self.colorButtonDiv = document.createElement("div");
	self.colorButtonDiv.setAttribute( "id", self.viewId + "ColorButtonDiv");
	self.view.append(self.colorButtonDiv);
	
	self.controlButtonDiv = document.createElement("div");
    self.controlButtonDiv.setAttribute( "id", self.viewId + "ControlButtonDiv");
    self.controlButtonDiv.setAttribute("class", "d-flex flex-wrap justify-content-around control-button-padding");
    self.view.append(self.controlButtonDiv);
	
	self.colorChangeDispatcher = colorChangeDispatcher;
	
	
	// create
	//self.buttonDiv = getDescendantById(self.view, "ColorButtonDiv");
	//self.savePaletteButton = getDescendantById(self.view, "savePaletteButton");
    //self.savePaletteButton.addEventListener('click', self.dummy);  // assignment needs to be _after_ the function declaration
	
	self.autoAddColors = autoAddColors;
	
	self.currentColor = null;
		
	self.buttons = new Array();
	
	
	
	
	self.addColorButtons = function(namedColorArray)
	{
		for (var i = 0; i < namedColorArray.length; i++)
		{
			self.addColorButton(namedColorArray[i]);
		}
	}
	
	
	self.addColorButton = function(namedColor)
    {
    	// create NamedColorButton with desired color as background, and name as tooltip
    	var namedColorButton = new NamedColorButton(namedColor);
    	
    	// set the onclick routine to our color-selected method
    	namedColorButton.button.onclick = self.setCurrentColorFromColorButton;
    	
    	// add to display and list of buttons
    	self.colorButtonDiv.append(namedColorButton.button);
    	self.buttons.push(namedColorButton);
    	
    	return namedColorButton;
    }
	
	
	self.removeColorButton = function(namedColorButton)
    {
    	// remove from display and list of buttons
    	self.colorButtonDiv.removeChild(namedColorButton.button);
    	self.buttons.splice(self.buttons.indexOf(namedColorButton), 1)
    }
	
	
	self.setCurrentColorFromColorButton = function(event)
    {
    	// event will be click from NamedColorButton.button
    	var button = event.target;
    	var namedColorButton = self.findNamedButton(button);
    	
    	if (namedColorButton != null)
    	{
    	    self.setSelectedButton(namedColorButton);
    	    self.currentColor = namedColorButton.namedColor;
    	
    	    // inform listeners
    	    self.colorChangeDispatcher.broadcastChange(self.currentColor);
    	}
		
    }
	
	
	self.getCurrentColor = function()
	{
		return self.currentColor;
	}
	
	
	self.findNamedButton = function(button)
	{
		for (var i = 0; i < self.buttons.length; i++)
		{
			if (button == self.buttons[i].button)
			{
				return self.buttons[i];
			}
		}
		
		return null;
	}
	
	
	
	// null deselects all...
	self.setSelectedButton = function(namedColorButton)
	{
		for (var i = 0; i < self.buttons.length; i++)
		{
			self.buttons[i].setSelected(false);
		}
		
		if (namedColorButton != null)
		{
			namedColorButton.setSelected(true);
		}
	}
	
	
	self.addCurrentColor = function()
	{
	    // first, see if we already have it
	    for (var i = 0; i < self.buttons.length; i++)
        {
            if (self.buttons[i].namedColor.equals(self.currentColor))
            {
                return;
            }
        }
        
        // if get here, don't have it; add
        var newButton = self.addColorButton(self.currentColor);
        self.setSelectedButton(newButton);
	}
	
	
	self.removeCurrentColor = function()
    {
        // see if we have it, and remove it if so
        for (var i = 0; i < self.buttons.length; i++)
        {
            if (self.buttons[i].namedColor.equals(self.currentColor))
            {
                self.removeColorButton(self.buttons[i]);
                return;
            }
        }
    }
	
	
	self.onColorChange = function(namedColor)
	{
		self.currentColor = namedColor;
		
		// deselect all buttons
		self.setSelectedButton(null);
		
		// see if have this color already, and select that button if so
		for (var i = 0; i < self.buttons.length; i++)
        {
            if (self.buttons[i].namedColor.equals(namedColor))
            {
                self.buttons[i].setSelected(true);
                return;
            }
        }
		
		// if get here, don't have it; add if we're adding colors automatically
		if (self.autoAddColors)
	    {
    		var newButton = self.addColorButton(namedColor);
    		newButton.setSelected(true);
	    }
		
	}
	
	self.colorChangeDispatcher.addChangeListener(self, self.onColorChange);
	
	
	
	/**
     * Instantiate Palette from XML document accessed by editor.
     * @param editor  XML document editor initialized with palette XML doc
     */
    self.initializeFromXml = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
    {
        var paletteElement = editor.getElement(parent, "color_palette");
        
        if (paletteElement == null)
            throw new XmlError("No palette found in file");
        
        var namedColorElements = editor.getElements(paletteElement, "named_color");
        
        for (var i = 0; i < namedColorElements.length; i++)
        {
            var name;
            if (namedColorElements[i].hasAttribute("color_name"))
            {
                name = namedColorElements[i].getAttribute("color_name");
            }
            else
            {
                name = "";
            }
            
            var valueString = namedColorElements[i].textContent;
            
            // value comes in as aarrggbb; change to just rrggbb with "#" prepended
            var rgbString = NamedColor.argbToColorString(valueString);
            var color = new Color(rgbString);
            
            var namedColor = NamedColor.getNamedColor(name, color);
            self.addColorButton(namedColor);
        }
        
        // set highlighting state
        self.onColorChange(self.currentColor);

    }

	
	
	self.writeXML = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
	{
		// create top-level color palette node
        var paletteElement = editor.appendElement(parent, "color_palette");
        
        for (var i = 0; i < self.buttons.length; i++)
        {
            var name = self.buttons[i].namedColor.name;
            var namedColor = self.buttons[i].namedColor;
            var element = editor.appendHexElement(paletteElement, "named_color", namedColor.getARGBString());
            element.setAttribute("color_name", name);
        }
	}
	
	
	self.generateXmlString = function()
    {
		// stuff for generating XML documents
	    var paletteDocumentRootElementName = "rosette_designer_palette";
	    var paletteDocumentHeaderComment = "Color palette data for use with RosetteDesigner application";
		
		
		// create DOM document with the list of colors represented, and populate it
        var xmlDoc = new Document();
        var docHeaderComment = paletteDocumentHeaderComment;
        xmlDoc.createComment(docHeaderComment);
        
        // add the standard XML processing instruction <?xml version="1.0" encoding="UTF-8"?>
        const processingInstruction = xmlDoc.createProcessingInstruction('xml', 'version="1.0" encoding="UTF-8"');
        xmlDoc.appendChild(processingInstruction);
        
        // create a "root" element for the document
        var docElementName = paletteDocumentRootElementName;
        var docElement = xmlDoc.createElement(docElementName);
        xmlDoc.appendChild(docElement);
        
        // just a convenience wrapper that provides methods for simultaneously creating, assigning value to, and appending Elements
        var editor = new XmlDocumentEditor(xmlDoc);
        
        // write out palette information to document
        self.writeXML(docElement, editor);
        
        // return the string contents of the doc
        var serializer = new XMLSerializer();
        var xmlString = serializer.serializeToString(xmlDoc);

        //console.log(xmlString);
        
        return xmlString;
    
    }
	
	
	
	// initialization
	{
	    // get last color, and highlight appropriately
	    self.currentColor = self.colorChangeDispatcher.getLastChange();
	    if (self.color != null)
	    {
	        self.onColorChange(self.currentColor);
	    }
	    
	    // create "add color" and "remove color" buttons if auto-add not set
	    if (!self.autoAddColors)
	    {
    	    addColorButton = document.createElement("button");
    	    addColorButton.setAttribute( "class", "btn-sm btn-secondary");
    	    addColorButton.setAttribute( "id", self.viewId + "AddColorButton");
    	    addColorButton.innerHTML = "Add current color";
    	    addColorButton.addEventListener('click', self.addCurrentColor);
    	    
    	    self.controlButtonDiv.append(addColorButton);
    	    
    	    
    	    var removeColorButton = document.createElement("button");
    	    removeColorButton.setAttribute( "class", "btn-sm btn-secondary");
    	    removeColorButton.setAttribute( "id", self.viewId + "RemoveColorButton");
    	    removeColorButton.innerHTML = "Remove current color";
    	    removeColorButton.addEventListener('click', self.removeCurrentColor);
    	    
    	    self.controlButtonDiv.append(removeColorButton);
	    }
	}

}