

Tile.DEFAULT_BACKGROUND_COLOR = NamedColor.getNamedColor("White", new Color("#ffffff"));
Tile.DEFAULT_GRID_COLOR = NamedColor.getNamedColor("Black", new Color("#000000"));



function Tile(numRows, numCols, backColor, gridColor)
{
	var self = this;
	
	// array maximum size
    self.maxNumRowsColumns = 80;

	// initialization
	self.numRows = numRows;
	self.numCols = numCols;
	
	self.backColor = backColor;
	self.gridColor = gridColor;
	
	self.element = new Array(numRows); 
    for (var row = 0; row < self.numRows; row++)
    {
        self.element[row] = new Array(self.numCols);
    	for (var column = 0; column < self.numCols; column++)
        {
        	self.element[row][column] = self.backColor;
        }
    }
	
	
    
    /*
     * Initialize from a given base tile, but applying the specified symmetry. Used
     * particularly in Rosette initialization
     */
	self.initializeFromBaseTile = function(baseTile, symmetry)
	{
		if ((self.numRows != baseTile.numRows) || (self.numCols != baseTile.numCols))
		{
			self.numRows = baseTile.numRows;
			self.numCols = baseTile.numCols;
			
			// create new 2d array
			self.element = new Array(self.numRows); 
		    for (var row = 0; row < self.numRows; row++)
		    {
		        self.element[row] = new Array(self.numCols);
		    }
		}
	    
		// populate element array
	    for (var row = 0; row < self.numRows; row++)
	    {
	        for (var column = 0; column < self.numCols; column++)
	        {
	        	var oddRow, oddColumn;
	            
	            // determine corresponding element position depending on symmetry
	            switch (symmetry)
	            {
	                case Rosette.BOOK_MATCH:
	                {
	                    oddRow = row;
	                    oddColumn = self.numCols - column - 1;
	                    break;
	                }
	                
	                case Rosette.INVERSION:
	                {
	                    oddRow = self.numRows - row - 1;
	                    oddColumn = column;
	                    break;
	                }
	                
	                case Rosette.BOOK_INVERSION:
	                {
	                    oddRow = self.numRows - row - 1;
	                    oddColumn = self.numCols - column - 1;
	                    break;
	                }
	                
	                case Rosette.SLIP_MATCH:
		            default:
	                {
	                    oddRow = row;
	                    oddColumn = column;
	                    break;
	                }
	                
	            }
	            
	            // set color appropriately
	            self.element[oddRow][oddColumn] = baseTile.element[row][column];
	        }
	    }
	
	}
	
	
    
    
	self.setRows = function(newNumRows)
	{
		self.setRowsCols(newNumRows, self.numCols);
	}
	
	
	self.setCols = function(newNumCols)
    {
        self.setRowsCols(self.numRows, newNumCols);
    }


	self.setRowsCols = function(newNumRows, newNumCols)
	{
	    // save current element for copying
	    var oldElement = self.element;
        var oldNumRows = self.numRows;
        var oldNumCols = self.numCols;
        
        // create a new array of appropriate size
        self.numRows = newNumRows;
        self.numCols = newNumCols;
		self.element = new Array(self.numRows); 
	    for (var row = 0; row < self.numRows; row++)
	    {
	        self.element[row] = new Array(self.numCols);
	    }
		
		// clear new array
		self.clear(self.backColor);
		
		// copy old array to new one
		var minNumRows = Math.min(oldNumRows, newNumRows);
		var minNumCols = Math.min(oldNumCols, newNumCols);
			
		for(var i = 0; i < minNumRows; ++i)
		{
			for(var j = 0; j < minNumCols; ++j)
			{
				self.element[i][j] = oldElement[i][j];
			}
		}
	}


	self.clear = function(backColor)
	{
	    for(var i = 0; i < self.numRows; ++i)
        {
            for(var j = 0; j < self.numCols; ++j)
            {
                self.element[i][j] = backColor;
            }
        }
	}
	
	
	self.shiftVertically = function(shift)
    {
        var oldElement = self.element;
        
        self.element = new Array(self.numRows); 
	    for (var row = 0; row < self.numRows; row++)
	    {
	        self.element[row] = new Array(self.numCols);
	    }
	    
        for(var i = 0; i < self.numRows; ++i)
            for(var j = 0; j < self.numCols; ++j)
                self.element[i][j] = oldElement[(i+self.numRows-shift)%self.numRows][j];
    }
    
    
    self.shiftHorizontally = function(shift)
    {
    	var oldElement = self.element;
        
    	self.element = new Array(self.numRows); 
	    for (var row = 0; row < self.numRows; row++)
	    {
	        self.element[row] = new Array(self.numCols);
	    }
	    
        for(var i = 0; i < self.numRows; ++i)
            for(var j = 0; j < self.numCols; ++j)
                self.element[i][j] = oldElement[i][(j+self.numCols-shift)%self.numCols];
    }
    
    
    
    /**
     * Instantiate Tile from XML document accessed by editor.
     * @param editor  XML document editor initialized with rosette XML doc
     */
    self.initializeFromXml = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
    {
        var tileElement = editor.getElement(parent, "tile");
        
        if (tileElement == null)
            throw new XmlError("No tile found in file");
        
        self.numRows = editor.getIntElementValue(tileElement, "num_rows", Rosette.DEFAULT_NUM_ROWS);
        self.numCols = editor.getIntElementValue(tileElement, "num_cols", Rosette.DEFAULT_NUM_COLUMNS);
        
        // get background color, with default of white if not specified in file
        var backgroundColorString = editor.getStringElementValue(tileElement, "background_color", Tile.DEFAULT_BACKGROUND_COLOR.getARGBString());
        var backgroundColorName = editor.getAttribute(tileElement, "background_color", "name", "");
        // convert from "aarrggbb" to "#rrggbb"
        backgroundColorString = NamedColor.argbToColorString(backgroundColorString);
        self.backColor = NamedColor.getNamedColor(backgroundColorName, new Color(backgroundColorString));
        
        // get grid color, with default of light gray if not specified in file
        var gridColorString = editor.getStringElementValue(tileElement, "grid_color", Tile.DEFAULT_GRID_COLOR.getARGBString());
        var gridColorName = editor.getAttribute(tileElement, "grid_color", "name", "");
        // convert from "aarrggbb" to "#rrggbb"
        gridColorString = NamedColor.argbToColorString(gridColorString);
        self.gridColor = NamedColor.getNamedColor(gridColorName, new Color(gridColorString));
        
        if ((self.numRows > self.maxNumRowsColumns) || (self.numCols > self.maxNumRowsColumns))
            throw new XmlError("Number of rows and columns must be less than" + self.maxNumRowsColumns);
        
        self.element = new Array(self.numRows); 
        for (var row = 0; row < self.numRows; row++)
        {
            self.element[row] = new Array(self.numCols);
        }
        
        for (var i = 0; i < self.numRows; i++)
        {
            var rowElement = editor.getElement(tileElement, "row_" + i.toString());
            for (var j = 0; j < self.numCols; j++)
            {
                var elementString = "element_row_" + i.toString() + "_col_" + j.toString();
                
                // get element color, with default of background color if not found
                var colorString = editor.getStringElementValue(rowElement, elementString, self.backColor.getARGBString());
                var name = editor.getAttribute(rowElement, elementString, "name", "");
                
                // convert from "aarrggbb" to "#rrggbb"
                colorString = NamedColor.argbToColorString(colorString);
                
                self.element[i][j] = NamedColor.getNamedColor(name, new Color(colorString));
            }
        }
        
    }
    
    
    
	/**
     * Generate XML representation of rosette tile
     * @return
     */
    self.writeXML = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
    {
        // create top-level rosette node
        var tileNode = editor.appendElement(parent, "tile");
        
        // put in data
        editor.appendIntElement(tileNode, "num_rows", self.numRows);
        editor.appendIntElement(tileNode, "num_cols", self.numCols);
        
        var backgroundColorElement = editor.appendStringElement(tileNode, "background_color", self.backColor.getARGBString());
        backgroundColorElement.setAttribute("name", self.backColor.getName());
        var gridColorElement = editor.appendStringElement(tileNode, "grid_color", self.gridColor.getARGBString());
        gridColorElement.setAttribute("name", self.gridColor.getName());
        
        for (var i = 0; i < self.numRows; i++)
        {
            // create new row node
            var rowNode = editor.appendElement(tileNode, "row_" + i.toString());
            for (var j = 0; j < self.numCols; j++)
            {
                var elementString = "element_row_" + i.toString() + "_col_" + j.toString();
                var tileColorElement = editor.appendStringElement(rowNode, elementString, self.element[i][j].getARGBString());
                tileColorElement.setAttribute("name", self.element[i][j].getName());
            }
        }
    }
    
}

