


function XmlDocumentEditor(document)
{
    
    self = this;
	
	self.doc = document;
        
    
    self.appendElement = function(/*Element*/ parent, /*String*/ tag)
    {
        var element = self.doc.createElement(tag);
        parent.appendChild(element);
        return element;
    }
    
    
    
    self.appendIntElement = function(/*Element*/ parent, /*String*/ tag, /*int*/ value)
    {
        var element = self.doc.createElement(tag);
        var textElement = self.doc.createTextNode(value.toString());
        element.appendChild(textElement);
        parent.appendChild(element);
        return element;
    }
    
    
    
    self.appendHexElement = function(/*Element*/ parent, /*String*/ tag, /*int*/ value)
    {
        var element = self.doc.createElement(tag);
        var textElement = self.doc.createTextNode(value.toString(16));
        element.appendChild(textElement);
        parent.appendChild(element);
        return element;
    }
    
    
    
    self.appendDoubleElement = function(/*Element*/ parent, /*String*/ tag, /*double*/ value)
    {
        var element = self.doc.createElement(tag);
        var textElement = self.doc.createTextNode(value.toString());
        element.appendChild(textElement);
        parent.appendChild(element);
        return element;
    }
    
    
    
    self.appendStringElement = function(/*Element*/ parent, /*String*/ tag, /*String*/ value)
    {
        var element = self.doc.createElement(tag);
        var textElement = self.doc.createTextNode(value);
        element.appendChild(textElement);
        parent.appendChild(element);
        return element;
    }
    
    
    
    self.getElements = function(/*Element*/ parent, /*String*/ tag)
    {
        var nodeList = parent.getElementsByTagName(tag);
        var numNodes = nodeList.length;
        var elementList = new Array(numNodes);
        
        for (var i = 0; i < numNodes; i++)
        {
            elementList[i] = nodeList.item(i);
        }
        
        return elementList;
    }
    
    
    
    self.getElement = function(/*Element*/ parent, /*String*/ tag)
    {
        var elementList = parent.getElementsByTagName(tag);
        
        // shouldn't be more than one element with given tag, but if so, return first
        // (or null if none)
        var element = elementList.item(0);
        
        return element;
    }
    
    
    
    self.getIntElementValues = function(/*Element*/ parent, /*String*/ tag)
    {
        
        var nodeList = parent.getElementsByTagName(tag);
        var numNodes = nodeList.length;
        var elementValues = new Array(numNodes);
        
        for (var i = 0; i < numNodes; i++)
        {
            var element = nodeList.item(i);
            
            if (element != null)
            {
                var valueString = element.textContent;
                elementValues[i] = parseInt(valueString);
            }
        }
        
        return elementValues;
    }
    
    
    
    self.getIntElementValue = function(/*Element*/ parent, /*String*/ tag, /*int*/ defaultValue)
    {
        var value;
        
        var elementList = parent.getElementsByTagName(tag);
        
        // shouldn't be more than one element with given tag, but if so, return first
        var element = elementList.item(0);
        
        if (element != null)
        {
            var valueString = element.textContent;
            value = parseInt(valueString);
            
            if (value == NaN)
            {
                // element not right; return default value
                return defaultValue;
            }
        }
        else // element not found; return default value
        {
            return defaultValue;
        }
        
        return value;
    }
    
    
    
    self.getHexElementValues = function(/*Element*/ parent, /*String*/ tag)
    {
        
        var nodeList = parent.getElementsByTagName(tag);
        var numNodes = nodeList.length;
        var elementValues = new Array(numNodes);
        
        for (var i = 0; i < numNodes; i++)
        {
            var element = nodeList.item(i);
            
            if (element != null)
            {
                var valueString = element.textContent;
                elementValues[i] = parseInt(valueString, 16);
            }
        }
        
        return elementValues;
    }
    
    
    
    self.getHexElementValue = function(/*Element*/ parent, /*String*/ tag, /*hex int*/ defaultValue)
    {
        var value;
        
        var elementList = parent.getElementsByTagName(tag);
        
        // shouldn't be more than one element with given tag, but if so, return first
        var element = elementList.item(0);
        
        if (element != null)
        {
            var valueString = element.textContent;
            value = parseInt(valueString, 16);
            
            if (value == NaN)
            {
                // element not right; return default value
                return defaultValue;            
            }
        }
        else // element not found; return default vale
        {
            return defaultValue;
        }
        
        return value;
    }
    
    
    
    self.getDoubleElementValues = function(/*Element*/ parent, /*String*/ tag)
    {
        
        var nodeList = parent.getElementsByTagName(tag);
        var numNodes = nodeList.length;
        var elementValues = new Array(numNodes);
        
        for (var i = 0; i < numNodes; i++)
        {
            var element = nodeList.item(i);
            
            if (element != null)
            {
                var valueString = element.textContent;
                elementValues[i] = parseFloat(valueString);
            }
        }
        
        return elementValues;
    }
    
    
    
    self.getDoubleElementValue = function(/*Element*/ parent, /*String*/ tag, /*double*/ defaultValue)
    {
        var value;
        
        var elementList = parent.getElementsByTagName(tag);
        
        // shouldn't be more than one element with given tag, but if so, return first
        var element = elementList.item(0);
        
        if (element != null)
        {
            var valueString = element.textContent;
            value = parseFloat(valueString);
            
            if (value == NaN)
            {
                // element not right; return default value
                return defaultValue;
            }
        }
        else // element not found; return default value
        {
            return defaultValue;
        }
        
        return value;
    }
    
    
    
    self.getStringElementValue = function(/*Element*/ parent, /*String*/ tag, /*string*/ defaultValue)
    {
        var value;
        
        var elementList = parent.getElementsByTagName(tag);
        
        // shouldn't be more than one element with given tag, but if so, return first
        var element = elementList.item(0);
        
        if (element != null)
        {
            value = element.textContent;
        }
        else // element not found; return default value
        {
            return defaultValue;
        }
        
        if (value == null)
            value = "";
        
        return value;
    }
    
    
    
    self.getAttribute = function(/*Element*/ parent, /*String*/ tag, /*String*/ attributeName, /*String*/ defaultValue)
    {
        var elementList = parent.getElementsByTagName(tag);
        
        // shouldn't be more than one element with given tag, but if so, return first
        var element = elementList.item(0);
        
        if ((element != null) && (element.hasAttribute(attributeName)))
        {
            return element.getAttribute(attributeName);
        }
        else
        {
            return defaultValue;
        }
    }
    
}